class Flat_graphics:
	def __init__(self):
		pass


class Rectangle(Flat_graphics):
	_width: int or float
	_height: int or float
	
	def __init__(self, width: int or float, height: int or float) -> None:
		super().__init__()
		if width == 0 or height == 0:
			raise ValueError("The width and height of the rectangle can't be 0.")
		elif isinstance(width, (int, float)) and isinstance(height, (int, float)):
			self._width = width
			self._height = height
		else:
			raise ValueError('The width and height of a rectangle must be a floating-point or integer number.')
	
	@property
	def area(self):
		width = self._width
		height = self._height
		return width * height
	
	def set_width(self, new_width: int or float) -> None:
		self._width = new_width
	
	def set_height(self, new_height: int or float) -> None:
		self._height = new_height
	
	@property
	def inspect(self) -> tuple:
		return self._width, self._height
	
	@property
	def circumference(self):
		return (self._width + self._height) * 2


class Triangle(Flat_graphics):
	_a: int or float
	_b: int or float
	_c: int or float
	
	def __init__(self, a: int or float, b: int or float, c: int or float):
		super().__init__()
		if a + b > c and a + c > b and b + c > a:
			self._a = a
			self._b = b
			self._c = c
		else:
			raise ValueError('The sum of any two sides of a triangle id greater than the third side.')
	
	@property
	def area(self):
		from math import sqrt
		s = (self._a + self._b + self._c) / 2
		return sqrt(s * (s - self._a) * (s - self._b) * (s - self._c))
	
	@property
	def inspect(self):
		return self._a, self._b, self._c
	
	def set_all(self, a, b, c):
		self._a = a
		self._b = b
		self._c = c
	
	@property
	def circumference(self):
		return self._a + self._b + self._c


class Circle(Flat_graphics):
	def __init__(self, radius, pi=3.14):
		super().__init__()
		if radius == 0:
			raise ValueError("The radius can't be 0.")
		elif not isinstance(radius, (int, float)):
			raise ValueError('The radius must be a floating-point or integer number.')
		elif not isinstance(pi, (int, float)):
			raise ValueError('Pi must be floating-point or integer numbers.')
		self._radius = radius
		self._diameter = radius * 2
		self._pi = pi
	
	def set_all(self, radius, pi):
		self._radius = radius
		self._pi = pi
	
	@property
	def area(self):
		return self._pi * self._radius ** 2
	
	@property
	def circumference(self):
		return self._pi * self._diameter
	
	def inspect(self):
		return self._radius, self._diameter, self._pi



class Fraction:
    def __init__(self, numerator: int, denominator: int) -> None:
        self.__numerator = numerator
        self.__denominator = denominator

    def __int__(self):
        return int(round(self.__numerator / self.__denominator, 0))

    def __str__(self):
        return f'{self.__numerator}/{self.__denominator}'

    def __repr__(self):
        return f'{self.__numerator}/{self.__denominator}'

    def __abs__(self):
        return abs(round(self.__numerator / self.__denominator, 2))

    def __bool__(self):
        return True if self.__numerator != 0 else False

    def __float__(self):
        return round(self.__numerator / self.__denominator, 2)

    def divide(self):
        if self.__numerator == self.__denominator:
            self.__numerator = 1
            self.__denominator = 1
        for i in range((self.__numerator if self.__numerator >= self.__denominator else self.__denominator) + 1, 2, -1):
            if self.__numerator % i == 0 and self.__denominator % i == 0:
                self.__numerator //= i
                self.__denominator //= i
                break

    def __add__(self, other):
        for i in range(self.__denominator if self.__denominator >= other.__denominator else other.__denominator,
                       self.__denominator if self.__denominator >= other.__denominator else other.__denominator,
                       self.__denominator * other.__denominator):
            if i % self.__denominator == 0 and i % other.__denominator == 0:
                b = i // self.__denominator
                c = i // other.__denominator
                self.__denominator = i
                break
        self.__numerator *= b
        d = other.__numerator * c
        self.__numerator += d
        self.divide()

    def __subtract__(self, other):
        b = 1
        c = 1
        for i in range(self.__denominator if self.__denominator >= other.__denominator else other.__denominator,
                       self.__denominator * other.__denominator):
            if i % self.__denominator == 0 and i % other.__denominator == 0:
                b = i // self.__denominator
                c = i // other.__denominator
                self.__denominator = i
                break
        self.__numerator *= b
        d = other.__numerator * c
        self.__numerator -= d
        self.divide()

    def __multiply__(self, other):
        self.__denominator *= other.__denominator
        self.__numerator *= other.__numerator
        self.divide()

    def __division__(self, other):
        t = other.__numerator
        other.__denominator = other.__numerator
        other.__numerator = t
        self.__multiply__(other)

    def pprint(self, output_type):
        if output_type == 0:
            print('''\
 {0:d} 
-{1:<s}-
 {2:d}\
            '''.format(self.__numerator, "-" * len(str(self.__numerator if self.__numerator >= self.__denominator else self.__denominator)), self.__denominator))
        elif output_type == 1:
            print(self.__str__())


def create_fraction_from_numerator_and_denominator(numerator: int, denominator: int) -> Fraction:
    return Fraction(numerator, denominator)


def create_fraction_from_float(num: float) -> Fraction:
    denominator = 10 ** len(str(num).split('.')[1])
    numerator = int(denominator * num)
    return Fraction(numerator, denominator)


def create_fraction_from_string(string: str) -> Fraction:
    denominator = 10 ** len(string.split('.')[1])
    numerator = int(denominator * float(string))
    return Fraction(numerator, denominator)


def create_fraction_from_parent_list(*parent_list) -> Fraction:
    return Fraction(parent_list[0], parent_list[1])


def create_fraction_from_parent_dict(**parent_dict) -> Fraction:
    return Fraction(parent_dict['numerator'], parent_dict['denominator'])



import decimal


def pi(n: int)->decimal.Decimal:
    decimal.setcontext(decimal.Context(prec=n, rounding=decimal.ROUND_HALF_DOWN))
    return decimal.Decimal(round(
        3.14159265358979323846264338327950288419716939937510582097494459230781640628620899862803, n))+decimal.Decimal(0)


decimal.setcontext(decimal.Context(prec=97, rounding=decimal.ROUND_HALF_DOWN))
pi_n = decimal.Decimal(
    3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421)+decimal.Decimal(0)
decimal.setcontext(decimal.Context(prec=2, rounding=decimal.ROUND_HALF_DOWN))
pi_approximate_rate = decimal.Decimal(round(22/7, 2))+decimal.Decimal(0)
pi05 = 3.14159
decimal.setcontext(decimal.Context(prec=6, rounding=decimal.ROUND_HALF_DOWN))
pi_density = decimal.Decimal(round(355/113, 6))+decimal.Decimal(0)
pi10 = 3.1415926535
pi15 = 3.141592653589793
decimal.setcontext(decimal.Context(prec=20, rounding=decimal.ROUND_HALF_DOWN))
pi20 = decimal.Decimal(3.14159265358979323846)+decimal.Decimal(0)
decimal.setcontext(decimal.Context(prec=25, rounding=decimal.ROUND_HALF_DOWN))
pi25 = decimal.Decimal(3.1415926535897932384626433)+decimal.Decimal(0)
decimal.setcontext(decimal.Context(prec=30, rounding=decimal.ROUND_HALF_DOWN))
pi30 = decimal.Decimal(3.141592653589793238462643383279)+decimal.Decimal(0)


def e(n: int)->decimal.Decimal:
    decimal.setcontext(decimal.Context(prec=n, rounding=decimal.ROUND_HALF_DOWN))
    return decimal.Decimal(round(
        2.71828182845904523536028747135266249775724709369995957496696762772407663035354759457138, n))+decimal.Decimal(0)


decimal.setcontext(decimal.Context(prec=97, rounding=decimal.ROUND_HALF_DOWN))
e_n = decimal.Decimal(
    2.7182818284590452353602874713526624977572470936999595749669676277240766303535475945713821785251)+decimal.Decimal(0)
e05 = 2.71828
e10 = 2.7182818284
e15 = 2.718281828459045
decimal.setcontext(decimal.Context(prec=20, rounding=decimal.ROUND_HALF_DOWN))
e20 = decimal.Decimal(2.71828182845904523536)+decimal.Decimal(0)
decimal.setcontext(decimal.Context(prec=25, rounding=decimal.ROUND_HALF_DOWN))
e25 = decimal.Decimal(2.7182818284590452353602874)+decimal.Decimal(0)
decimal.setcontext(decimal.Context(prec=30, rounding=decimal.ROUND_HALF_DOWN))
e30 = decimal.Decimal(2.718281828459045235360287471352)+decimal.Decimal(0)



from src.mathextend.Flat_graphics import *

INFINITE_GREATNESS = 10

for i in range(5):
    INFINITE_GREATNESS **= 10


class Three_dimensional_graphics:
    def __init__(self):
        pass


class Cylinder(Three_dimensional_graphics):
    def __init__(self, underside, height):
        super().__init__()
        if isinstance(underside, (Circle, Triangle, Rectangle)) and height and isinstance(height, (int, float)):
            self._Underside = underside
            if height >= INFINITE_GREATNESS:
                raise ValueError('The height is too high.')
            else:
                self._height = height
        else:
            raise ValueError('The height must be a floating-point or integer number and not be 0.')
    
    @property
    def volume(self):
        return self._Underside.area * self._height
    
    @property
    def surface_area(self):
        return 2 * self._Underside.area + self._Underside.circumference * self._height
    
    def set_all(self, underside, height):
        if isinstance(underside, Rectangle) and height and isinstance(height, (int, float)):
            self._Underside = underside
            if height >= INFINITE_GREATNESS:
                raise ValueError('The height is too high.')
            else:
                self._height = height
        else:
            raise ValueError('The height must be a floating-point or integer number and not be 0.')
    
    @property
    def inspect(self) -> tuple:
        return self._Underside, self._height


class Sphere(Circle):
    def __init__(self, radius, pi=3.14):
        super().__init__(radius, pi)
    
    def set_all(self, radius, pi):
        super().set_all(radius, pi)
    
    def inspect(self):
        super().inspect()
    
    @property
    def volume(self):
        return super().area * super()._radius
    
    @property
    def surface_area(self):
        return super().area ** 2
    
    @property
    def inspect(self) -> tuple:
        return super().inspect()


class Cone(Three_dimensional_graphics):
    def __init__(self, underside, height):
        super().__init__()
        if isinstance(underside, Circle):
            if height == 0:
                raise ValueError("Height can't be 0.")
            elif height >= INFINITE_GREATNESS:
                raise ValueError('The height is too high.')
            else:
                self._underside = underside
                self._height = height
        else:
            raise TypeError('Underside must be Circle.')
    
    def set_all(self, underside, height):
        self.__init__(underside, height)
    
    @property
    def volume(self):
        return Cylinder(self._underside, self._height).volume / 3
    
    @property
    def inspect(self) -> tuple:
        return self._underside, self._height

